package cn.sh.stc.sict.theme.hphy.controller.mp;


import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import cn.sh.stc.sict.cloud.common.core.util.NumberUtil;
import cn.sh.stc.sict.cloud.common.core.util.R;
import cn.sh.stc.sict.cloud.common.security.util.SecurityUtils;
import cn.sh.stc.sict.cloud.upms.dto.CurrentUser;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientBase;
import cn.sh.stc.sict.theme.hphy.model.HphyPatientRelation;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientBaseService;
import cn.sh.stc.sict.theme.hphy.service.HphyPatientRelationService;
import cn.sh.stc.sict.theme.hphy.vo.PatientRelationDTO;
import cn.sh.stc.sict.theme.hphy.vo.PatientRelationVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.io.Serializable;
import java.util.List;

/**
 * 患者-家庭成员关系表(HphyPatientRelation)表控制层
 *
 * @author F_xh
 * @since 2022-10-10 14:41:04
 */
@Slf4j
@RestController
@AllArgsConstructor
@Api(tags = "[C]患者-家庭成员关系表")
@RequestMapping("/hphyPatientRelation")
public class HphyPatientRelationController {
    /**
     * 服务对象
     */
    private final HphyPatientRelationService hphyPatientRelationService;
    private final HphyPatientBaseService hphyPatientBaseService;


    @ApiOperation("获取自己的家庭成员")
    @GetMapping("/list")
    public R list() {
        CurrentUser current = SecurityUtils.getCurrentUser();
        HphyPatientBase base = hphyPatientBaseService.getByOpenId(current.getOpenId());
        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            return new R();
        }
        List<PatientRelationVO> result = hphyPatientRelationService.listPatientRelation(base.getId());

        return new R(result);
    }

    @ApiOperation("新增家庭成员")
    @PostMapping
    @Transactional(rollbackFor = Exception.class)
    public R insert(@RequestBody PatientRelationDTO dto) {
        HphyPatientBase base = hphyPatientBaseService.getCurrentBase();
        if (ObjectUtil.isNull(base) || NumberUtil.isNullOrZero(base.getId())) {
            return new R().error("当前登陆人信息异常");
        }
        if (StrUtil.isBlank(dto.getPatient().getName())) {
            return new R().error("姓名不可为空！");
        }
        if (StrUtil.isBlank(dto.getPatient().getCertId())) {
            return new R().error("身份证号不可为空！");
        }
        if (StrUtil.isBlank(dto.getRelation().getRelationName())) {
            return new R().error("关系不可为空！");
        }
        if (StrUtil.isBlank(dto.getPatient().getPhone())) {
            return new R().error("手机号不可为空！");
        }
        hphyPatientBaseService.savePatient(dto.getPatient());
        dto.getRelation().setSourcePatientId(base.getId());
        dto.getRelation().setTargetPatientId(dto.getPatient().getId());
        LambdaQueryWrapper<HphyPatientRelation> wrapper = Wrappers.lambdaQuery();
        wrapper.and(i -> i.eq(HphyPatientRelation::getSourcePatientId, base.getId()).eq(HphyPatientRelation::getTargetPatientId, dto.getPatient().getId()))
                .or().and(i -> i.eq(HphyPatientRelation::getSourcePatientId, dto.getPatient().getId()).eq(HphyPatientRelation::getTargetPatientId, base.getId()));

        hphyPatientRelationService.saveOrUpdate(dto.getRelation(), wrapper);
        return new R();
    }

    @ApiOperation("删除关系")
    @DeleteMapping("/{relationId}")
    public R delete(@PathVariable Long relationId) {
        HphyPatientRelation relation = hphyPatientRelationService.getById(relationId);
        if (ObjectUtil.isNull(relation) || NumberUtil.isNullOrZero(relation.getId())) {
            return new R().error("未找到待删除项！");
        }
        hphyPatientRelationService.removeById(relationId);
        return new R();
    }

}